# -*- coding: utf-8 -*-
"""
Note that this duplicates anaconda_platform.common (without type annotations) because it needs to
support Python 2.7
"""
import copy
import logging
import os
import os.path

import jsonschema

from anaconda_enterprise.cli.internal import yaml

log = logging.getLogger(__name__)

variables = {
    'system_dir': '/etc',
    'config_dir': 'anaconda-platform',
}

platform_defaults = (
    '{source_dir}/schemas/v{version}/defaults/{component_file}', )

system_wide = ('{system_dir}/{config_dir}/{component_file}', )

user_local_component = '{home_dir}/{config_dir}/{component_file}'
user_local = (user_local_component, )

conda_env = ('{env_dir}/{config_dir}/{component_file}', )

# When the config schema changes, we need to be able
# to convert configuration in older schemas, so that the code
# base can refer strictly to the latest version

# These constants represent the schema versions we are aware of
SCHEMA_0 = '0'
SCHEMA_1 = '1'
SCHEMA_LATEST = SCHEMA_1
# Dictionary of functions that transform configuration dicts
# of one version to another.
UPGRADES = {}


def register_upgrade(from_version, to_version):
    """
    A decorator to add a conversion function that
    upgrades a configuration dict from version.

    upgrade function should have signature: Dict -> Dict

    The conversion function may mutate the passed in config.
    The conversion function does not need to mutate the `version` field of the dict.
    """
    def decorator(fn):
        def inner(cfg, *args, **kwargs):
            from_cfg = copy.deepcopy(cfg)
            to_cfg = fn(from_cfg, *args, **kwargs)
            to_cfg[ConfigManager.VERSION] = to_version
            return to_cfg

        # This may need to be smarter in the future with versions...
        # but it's probably easiest to treat them like migrations with a linear history.

        # We won't run into any issues with the dictionary keys if we write it that way.
        assert from_version not in UPGRADES, \
            'Upgrade from_version {} already exists'.format(from_version)
        UPGRADES[from_version] = inner

        return inner

    return decorator


class ConfigurationError(Exception):
    """A problem exists with the configuration"""


class ConfigManager(object):

    # Configuration Path Template Variables
    COMPONENT_FILE = 'component_file'
    SOURCE_DIR = 'source_dir'
    ENV_DIR = 'env_dir'
    HOME_DIR = 'home_dir'
    VERSION = 'version'

    def __init__(self, component, version):
        self.component = component.split('.')
        self.version = version
        self._data = {}  # current configuration
        self.schema = None
        self.variables = variables.copy()
        self.variables[self.VERSION] = version
        self.variables[self.COMPONENT_FILE] = '{}.yml'.format(component)
        self.variables[self.SOURCE_DIR] = self.source_dir
        self.variables[self.HOME_DIR] = os.getenv(
            'BINSTAR_CONFIG_DIR') or os.path.expanduser('~/.anaconda')

        prefix = os.environ.get('CONDA_PREFIX')
        if prefix is not None:
            self.variables[self.ENV_DIR] = '{}/etc'.format(prefix)

    def validate(self, config=None):
        if config is None:
            config = self._data

        if self.schema:
            log.debug('Validating configuration: %s', config)
            try:
                jsonschema.validate(config,
                                    self.schema,
                                    format_checker=jsonschema.FormatChecker())
            except jsonschema.exceptions.ValidationError as exc:
                msg = "'%s', %s" % ('.'.join(exc.path), exc.message)
                err = 'validating configuration %s' % msg
                log.debug(err)
                raise ConfigurationError(err)

    @property
    def source_dir(self):
        """Return the directory where this source file lives."""
        return os.path.dirname(os.path.abspath(__file__))

    @property
    def schema_path(self):
        """Return the path to the configuration schema file shipped with the software."""
        return '{source_dir}/schemas/v{version}/config.yml'.format(
            **self.variables)

    @property
    def token_path(self):
        return '{home_dir}/{config_dir}/tokens.json'.format(**self.variables)

    @property
    def user_local_save_path(self):
        return user_local_component.format(**self.variables)

    def load_yaml_string(self, config_str, path, conf_dict=None):
        """Parse a YAML string and validate it against our configuration schema."""
        try:
            result = yaml.load(config_str)
        except Exception:
            log.error('Error parsing YAML configuration file %s', path)
            raise

        if not result:
            log.warning('Empty config YAML in file %s', path)
            return

        try:
            upgraded_result = self.upgrade(result)
            if result != upgraded_result:
                result['version'] = upgraded_result['version']
                if result != upgraded_result:
                    log.warning(
                        "File: %s has deprecated configuration.\n"
                        "Configuration was updated on the fly but not modified.",
                        path)
            result = upgraded_result
            self.validate(result)
        except ConfigurationError as exc:
            log.error('Invalid configuration in file %s: %s', path, exc)
            raise
        conf = conf_dict if conf_dict is not None else self._data
        recursive_update(conf, result)

    def load_yaml_file(self, path, conf_dict=None):
        try:
            log.info('Reading configuration from %s', path)
            with open(path) as f:
                config_str = f.read()
        except Exception:
            # This will catch problems with opening or reading the file.
            log.error('Error reading configuration file %s', path)
            raise

        self.load_yaml_string(config_str, path, conf_dict)
        return conf_dict

    def load_dict(self, d):
        log.info('Loading configuration from dict:')
        for key, value in d.items():
            log.info('  %s: %r', key, value)
            self[key] = value
        self.validate()

    def load_files(self):
        """Load all the YAML config files in the search path."""
        for path in self.find_config_files():
            self.load_yaml_file(path)

    def load_schema(self):
        """
        Load the config file schema.

        The schema is a YAML file with JSONSchema semantics.
        If the file does not exist, schema validation will be bypassed.
        """

        path = self.schema_path

        if os.path.exists(path):
            log.info('Loading config schema file %s', path)
            with open(path) as f:
                self.schema = yaml.load(f)
        else:
            log.warning("Can't find config schema file %s", path)

    def get_site(self):
        site = self['default_site']
        if site not in self['sites']:
            raise ValueError(
                'Site: {site} not listed in sites option'.format(site=site))
        return site

    def get_user_config(self):
        path = self.user_local_save_path
        if os.path.exists(path):
            return self.load_yaml_file(path, {})
        else:
            return {'version': SCHEMA_LATEST}

    def new_config(self, key, value):
        # If key is `sites.example.url` and value `'http://repo:8089/api/`
        # This will create something like: {sites: {'example': {'url': 'http://repo:8089/api/}}}
        conf = {}
        self._update_tree(key, value, conf)
        self.validate(conf)
        return conf

    def merge_configs(self, target, new_):
        recursive_update(target, new_)
        return target

    def dump(self, conf_dict=None, file_=None):
        """Dumps the configuration to disk.

        If `file_` should be an object that has the `.write()`
        interface. If None is provided, this will write to the
        `user_local` path.

        """
        if conf_dict is None:
            conf_dict = self._data

        if file_ is None:
            path = self.user_local_save_path
            log.info('Saving configuration to %s', path)
            dirname = os.path.dirname(path)
            if not os.path.exists(dirname):
                os.makedirs(dirname)
            with open(path, 'w') as file_:
                yaml.dump(conf_dict, file_)
        else:
            yaml.dump(conf_dict, file_)

    def delete_token(self):
        """Delete the token file at self.token_path.

        Output:

        bool: True if files exists (and has been removed); false otherwise
        """
        if os.path.exists(self.token_path):
            os.remove(self.token_path)
            return True
        else:
            return False

    def __getitem__(self, key):
        """
        Return a single configuration value or part of the tree.

        Given a configuration key as a dot-separated path,
        traverse the configuration tree to find the value.
        This will first check in the component-specific config,
        followed by the global config.

        If the key can't be found, raises KeyError.
        """
        for prefix in self.prefixes(self.component):
            try:
                return self.find_config_value(self._data, prefix + key)
            except KeyError:
                continue

        # Not found anywhere in the hierarchy.
        # re-raise with the full original key
        raise KeyError(key)

    def _update_tree(self, key, value, root):
        keypath = key.split('.')
        cfg = root

        for k in keypath[:-1]:
            cfg = cfg.setdefault(k, {})

        cfg[keypath[-1]] = value

    def __setitem__(self, key, value):
        """
        Set a single configuration value or part of the tree.

        Given a configuration key as a dot-separated path,
        this will place `value` in the config variable specified
        by that path. Intervening path components will be created
        if necessary. Existing values will remain in place, except for
        the final (leaf) value which will be replaced by `value`.
        """
        self._update_tree(key, value, self._data)

    #
    # INTERNALS
    #
    @property
    def config_path_templates(self):
        """Return a tuple of templates which should be used to generate the search paths"""
        paths = platform_defaults + system_wide
        if self.ENV_DIR in self.variables:
            paths = paths + conda_env
        paths = paths + user_local
        return paths

    def get_search_path(self):
        """
        Return the list of eligible config file paths.

        Files in this list are the ones that will be read, if they exist.
        """
        return [
            path.format(**self.variables)
            for path in self.config_path_templates
        ]

    def find_config_files(self):
        """Return the list of config files that will be read."""
        return filter(os.path.exists, self.get_search_path())

    @staticmethod
    def prefixes(keypath):
        for i in reversed(range(len(keypath) + 1)):
            prefix = '.'.join(keypath[:i])
            yield prefix + '.' if prefix else ''

    @staticmethod
    def find_config_value(config, key):
        keypath = key.split('.')

        for k in keypath:
            config = config[k]
        return config

    def upgrade(self, conf):
        """Convert a configuration dict *conf* to the version of this config manager"""
        # We default to an unspecified schema version
        version = conf.get('version', SCHEMA_0)
        if version == self.version:
            return conf
        elif version in UPGRADES:
            old_config_manager = ConfigManager('.'.join(self.component),
                                               version)
            old_config_manager.load_schema()
            old_config_manager.validate(conf)
            conf = UPGRADES[version](conf)
            # recursively upgrade, if the version is multiple versions old
            # TODO: test this more thoroughly when there's more than a single upgrade
            return self.upgrade(conf)
        else:
            raise ConfigurationError(
                "Unrecognized configuration version: {0}".format(version))

    @staticmethod
    @register_upgrade(SCHEMA_0, SCHEMA_1)
    def upgrade_v0_v1(cfg):
        warn = False
        if 'verify_ssl' in cfg:
            cfg['ssl_verify'] = cfg.pop('verify_ssl')
            warn = True
        for site in cfg.get('sites', []):
            if 'verify_ssl' in cfg['sites'][site]:
                cfg['sites'][site]['ssl_verify'] = cfg['sites'][site].pop(
                    'verify_ssl')
                warn = True

        if warn:
            log.warning(
                'Configuration option "verify_ssl" deprecated as of 5.1.0;'
                ' use "ssl_verify" instead')

        return cfg


def recursive_update(target, new):
    """Recursively update one dictionary using another.

    None values will delete their keys.
    """
    # This portion:
    # Copyright (c) IPython Development Team.
    # Distributed under the terms of the Modified BSD License.
    for k, v in new.items():
        if isinstance(v, dict):
            if k not in target:
                target[k] = {}
            recursive_update(target[k], v)
            if not target[k]:
                # Prune empty subdicts
                del target[k]

        elif v is None:
            target.pop(k, None)

        else:
            target[k] = v


config = ConfigManager('cli', SCHEMA_LATEST)


def load_config():
    config.load_schema()
    config.load_files()
    return config



# -*- coding: utf-8 -*-

# Copyright (c) 2014-2015 Markus Unterwaditzer & contributors
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is furnished to do
# so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import logging
import sys

import click

_ctx = click.get_current_context

LOGGER_KEY = __name__ + '.logger'
DEFAULT_LEVEL = logging.INFO

PY2 = sys.version_info[0] == 2

if PY2:
    text_type = unicode  # noqa
else:
    text_type = str


def _meta():
    return _ctx().meta.setdefault(LOGGER_KEY, {})


class ColorFormatter(logging.Formatter):
    colors = {
        'error': dict(fg='red'),
        'exception': dict(fg='red'),
        'critical': dict(fg='red'),
        'debug': dict(fg='blue'),
        'warning': dict(fg='yellow')
    }

    def format(self, record):
        record.message = record.getMessage()
        if self.usesTime():
            record.asctime = self.formatTime(record, self.datefmt)
        try:
            s = self._fmt % record.__dict__
        except UnicodeDecodeError as e:
            # Issue 25664. The logger name may be Unicode. Try again ...
            try:
                record.name = record.name.decode('utf-8')
                s = self._fmt % record.__dict__
            except UnicodeDecodeError:
                raise e

        level = record.levelname.lower()
        if level in self.colors:
            prefix = click.style('{}: '.format(level), **self.colors[level])
            if not PY2 and isinstance(s, bytes):
                s = s.decode(sys.getfilesystemencoding(), 'replace')
            elif not isinstance(s, (text_type, bytes)):
                s = str(s)
            s = '\n'.join(prefix + x for x in s.splitlines())

        if record.exc_info:
            # Cache the traceback text to avoid converting it multiple times
            # (it's constant anyway)
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            if s[-1:] != "\n":
                s = s + "\n"
            try:
                s = s + record.exc_text
            except UnicodeError:
                # Sometimes filenames have non-ASCII chars, which can lead
                # to errors when s is Unicode and record.exc_text is str
                # See issue 8924.
                # We also use replace for when there are multiple
                # encodings, e.g. UTF-8 for the filesystem and latin-1
                # for a script. See issue 13232.
                s = s + record.exc_text.decode(sys.getfilesystemencoding(),
                                               'replace')
        return s


class ClickHandler(logging.Handler):
    def emit(self, record):
        try:
            msg = self.format(record)
            level = record.levelname.lower()
            err = level in ('warning', 'error', 'exception', 'critical')
            click.echo(msg, err=err)
        except Exception:
            self.handleError(record)


_default_handler = ClickHandler()
_default_handler.formatter = ColorFormatter()


def _normalize_logger(logger):
    if not isinstance(logger, logging.Logger):
        logger = logging.getLogger(logger)
    return logger


def basic_config(logger=None):
    '''Set up the default handler (:py:class:`ClickHandler`) and formatter
    (:py:class:`ColorFormatter`) on the given logger.'''
    logger = _normalize_logger(logger)
    logger.handlers = [_default_handler]
    logger.propagate = False

    return logger



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
import json
import os.path
import sys
from collections import defaultdict

import click

from anaconda_enterprise.cli import deconstruct
from anaconda_enterprise.cli.api import create_client
from anaconda_enterprise.cli.internal import yaml

from . import upload

ERROR_ = click.style('ERROR', fg='red')
WARNING_ = click.style('WARNING', fg='yellow')
INFO_ = click.style('INFO', fg='green')

META = 'meta.json'


@click.group(name='admin', hidden=True)
def main():
    """This command group includes several commands reserved primarily
       for use within the AE5 repository subsystem. For this reason,
       this group is hidden by default from the help output.
    """


def validate_extra_attributes(ctx, param, value):
    try:
        content = yaml.load(value)
        if not isinstance(content, dict):
            raise click.BadParameter('extra attributes needs to be an object')
        return content
    except ValueError as exc:
        raise click.BadParameter(exc.args[0])


@main.command()
@click.argument('installer', type=click.Path(exists=True), nargs=-1)
@click.option('--extra-attributes',
              default='{}',
              callback=validate_extra_attributes)
@click.pass_obj  # object in context is the Configuration Manager
def upload_installer(config_manager, installer, extra_attributes):
    """Upload an installer/parcel for distribution
    """
    client = create_client(config_manager)

    upload_count = 0
    for pkg in installer:
        pkg_name = os.path.basename(pkg)
        size = os.path.getsize(pkg)

        try:
            click.echo('Uploading {0} ({1} bytes)'.format(pkg_name, size),
                       nl=False)
            meta = deconstruct.extract(pkg)

            spec_id = extra_attributes.pop("env_spec_id", None)
            meta['extra-attributes'] = extra_attributes
            upload_info = client.stage_installer(meta, env_spec_id=spec_id)

            with open(pkg, 'rb') as fd:
                client.upload_file(upload_info, fd)

            upload_count += 1
            click.echo('.')
        except Exception as exc:
            click.echo('\n{0}: {1}'.format(ERROR_, exc))

    if upload_count == len(installer):
        click.echo('Complete. Uploaded {0} of {1}'.format(
            upload_count, len(installer)))
    else:
        click.echo('Failed. Uploaded {0} of {1}'.format(
            upload_count, len(installer)))
        sys.exit(1)


@main.command()
@click.argument('url')
@click.pass_obj  # object in context is the Configuration Manager
def fetch_resource(config_manager, url):
    """Perform arbitrary calls to the AE5 API
    """
    client = create_client(config_manager)
    print(json.dumps(client.fetch_resource(url), indent=2))


@main.command()
@click.argument('env_spec_id')
@click.argument('lock', type=click.Path(exists=True), nargs=1)
@click.option('--extra-attributes',
              default='{}',
              callback=validate_extra_attributes)
@click.pass_obj  # object in context is the Configuration Manager
def store_env_spec_lock(config_manager, env_spec_id, lock, extra_attributes):
    """Upload a resolved environment spec
    """
    client = create_client(config_manager)
    with open(lock) as fd:
        lock = json.load(fd)
    client.store_env_spec_lock(env_spec_id, lock, extra_attributes)


@main.command(name="import")
@click.argument("dump", type=click.Path(exists=True), nargs=-1)
@click.option("--dry-run", default=False)
@click.pass_context
def import_(ctx, dump, dry_run):
    """Import an Anaconda Repository v4 tree

    Keywords:

    ``dump`` -- A list of directories that are the result of using
    ``anaconda-server-admin export``

    ``dry_run`` -- If set to True prints a YAML to stdout which
    includes channels, file to channel mapping and the user's
    channels.

    Functionality:

    For every user in a repo4 it will create a new channel per label
    the user has, using the username as a prefix.

    An example:

    Say the user ``anaconda-user-1`` has the following packages:

    * ``moto-0.4.31-2.tar.bz2`` with label main
    * ``pysocks-1.6.6-py35_0.tar.bz2`` with label test

    For this user, the script will create the following channels:

    * ``anaconda-user-1`` with file ``moto-0.4.31-2.tar.bz2``
    * ``anaconda-user-1/test`` with file ``pysocks-1.6.6-py35_0.tar.bz2``

    The label "main" is dropped in preference of just keeping the
    username as channel name.

    For every organization we have to add the groups to the channel
    names.

    Say ``anaconda-organization`` has a group called "Devs" and there
    are packages there such as ``xz-5.2.2-1.tar.bz2`` which has label
    test. This script will create the following channels:

    * ``anaconda-organization`` -- This will have everything from the Owner group

    * ``anaconda-organization/Devs`` -- This will have everything that's
      available for the Dev group

    * ``anaconda-organization/Devs/test`` -- This will have everything
      that's in the Dev group with label test.

    So for an organization, group, and label a new channel is
    created. Just that the label "main" and group "Owner" are dropped
    in preference of making names shorter.
    """

    # here we keep channels and files
    _import_data = []

    # The following two vars are going to be used to join information regarding
    # which channels should be shared with the users.
    _user_acl_data = {}
    _chan_prefix_and_labels = defaultdict(set)

    # In v4 each file in a package can have a set of labels. Those labels should be changed
    # to channels. For a user that's *not* an organization, that is pretty simple.
    # We create a channel for each pair (user, label) e.g: user/main, user/test ("/" is valid in a
    # channel name). The complicated bit comes with organizations that have groups of users.

    # For an org. we should create a channel for each triplet (organization, group, label) and
    # the files that go in there depend on the on the group and label they have.

    for path_ in dump:

        meta_path = os.path.join(path_, META)
        if not os.path.exists(meta_path):
            click.echo("{0}: {1}".format(
                WARNING_, "This directory does not contain user info"))
            continue

        # Open the meta.json that has the user information
        with open(meta_path) as user_meta:
            user_info = json.load(user_meta)

        username = user_info["username"]

        # This holds the data we're going to import.
        account = {'username': username, 'files': [], 'channels': []}

        # Let's grab the group information of the user
        if user_info["user_type"] == "user":
            # We have to give permission to this user to the channels
            # which are prefixed with the following:
            extra_channels_for_user = [
                "{username}/{group}".format(username=g["owner"]["username"],
                                            group=g["name"])
                for g in user_info["groups"]
            ]
            # ^ The labels for the "extra channels" above are in the
            # organization account information (the meta.json for that user).
            # We use this data to match which users are allowed to which channels.

            _user_acl_data[username] = extra_channels_for_user

        # Fetch all the packages for this user from the meta.json
        for package in user_info["packages"]:

            package_info = {}

            path_to_packages = os.path.join(path_,
                                            package["package_output_dir"])
            with open(os.path.join(path_to_packages, META)) as package_meta:
                package_info = json.load(package_meta)

            channel_prefixes = [username]
            if user_info["user_type"] == 'org':
                channel_prefixes += [
                    "{username}/{group}".format(
                        username=g["owner"]["username"], group=g["name"])
                    for g in package_info["groups"]
                ]

            for file_ in package_info['files']:
                filepath = file_["dump_filename"]
                dist_type = file_["distribution_type"]

                # Ignore those that are not conda packages
                if dist_type != "conda":
                    click.echo(
                        "{0}: {1} is a {2} and will not be uploaded.".format(
                            WARNING_, filepath, dist_type))
                    continue

                # Ignore those that don't have labels.
                if not file_["labels"]:
                    click.echo(
                        "{0}: {1} has no labels and will not be uploaded".
                        format(WARNING_, filepath))
                    continue

                file_attrs = {}
                file_attrs["path"] = "{}".format(
                    os.path.join(path_to_packages, filepath))
                file_attrs["basename"] = file_[
                    "basename"]  # used to copy later on

                file_channels = []
                for p in channel_prefixes:
                    chan_names = [
                        chan_name(p, label) for label in file_["labels"]
                    ]
                    # we need this join the user's groups with the channels created
                    # for an organization.
                    if user_info["user_type"] == 'org':
                        _chan_prefix_and_labels[p].update(chan_names)
                    file_channels.extend(chan_names)

                file_attrs["channels"] = file_channels

                account["files"].append(file_attrs)

                for chan in file_channels:
                    if chan not in account["channels"]:
                        account["channels"].append(chan)
        _import_data.append(account)

    # Join user/channel data on the "prefix"
    _join_user_chans = {}
    for key, prefixes in _user_acl_data.items():
        users_chan = _join_user_chans.setdefault(key, [])
        for p in prefixes:
            for chan in _chan_prefix_and_labels[p]:
                users_chan.append(chan)

    if dry_run:
        yaml.dump({
            "accounts": _import_data,
            "sharing": _join_user_chans
        }, sys.stdout)
        return

    # for each account create the channels, and upload each file
    # to the corresponding channel, and share with the user or groups.
    client = create_client(ctx.obj)

    for account in _import_data:
        files = account['files']
        username = account['username']

        if not files:
            continue

        # create the channels before uploading
        channels = account['channels']
        for chan in channels:
            if client.get_channel(chan) is None:
                click.echo("{0}: Creating {1}".format(INFO_, chan))
                chan_obj = client.create_channel(chan)
                click.echo("{0}: Sharing {1} with user {2} as rw".format(
                    INFO_, chan, username))
                client.share_channel(chan_obj, username, 'users', 'rw')

        for file_ in account['files']:
            # Upload to a channel
            chan = file_["channels"][0]
            click.echo("{0}: uploading {1}".format(INFO_, file_['path']))
            try:
                upload._upload(ctx.obj,
                               channel=chan,
                               package=[file_['path']],
                               ignore_name=True)
            except Exception as e:
                click.echo("\n{0}: {1} from file: {2}".format(
                    ERROR_, e, file_["path"]))
                continue

            # Copy to the rest of the channels
            src_chan = chan
            src_subdir, src_filename = file_["basename"].rsplit("/", 1)
            for chan in file_['channels'][1:]:
                click.echo("{0}: copying {1} to {2}".format(
                    INFO_, file_['path'], chan))
                dest_chan = client.get_channel(chan)
                client.copy_pkg_file(src_chan, src_subdir, src_filename,
                                     dest_chan["id"])

    for username, channels in _join_user_chans.items():
        if not channels:
            continue
        for chan in channels:
            chan_obj = client.get_channel(chan)
            click.echo("{0}: Sharing {1} with user {2} as r".format(
                INFO_, chan, username))
            client.share_channel(chan_obj, username, 'users', 'r')


def chan_name(prefix, label):
    """Generate a channel name from a prefix and label

    Keywords:

    * prefix -- a str, which is the username or username/group
    * label -- a str, which is the label of a package in Repository v4.
    """
    owners_prefix = "/Owners"

    if owners_prefix in prefix:
        channel_name = prefix[:len(owners_prefix)]
    else:
        channel_name = prefix

    if label != 'main':
        channel_name += "/" + label
    return channel_name



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
import io

import pytest

from anaconda_enterprise.cli.config import (ConfigManager, platform_defaults, system_wide,
                                            user_local, conda_env, ConfigurationError,
                                            SCHEMA_LATEST, SCHEMA_1)


def test_remove_conda_env_templates_without_conda_prefix(monkeypatch):
    monkeypatch.delenv('CONDA_PREFIX')
    mgr = ConfigManager('cli', SCHEMA_LATEST)
    assert mgr.config_path_templates == platform_defaults + system_wide + user_local


def test_make_sure_conda_env_templates_are_added_with_conda_prefix(monkeypatch):
    monkeypatch.setenv('CONDA_PREFIX', 'test')  # just to make sure it's set
    mgr = ConfigManager('cli', SCHEMA_LATEST)
    assert mgr.config_path_templates == platform_defaults + system_wide + conda_env + user_local
    assert mgr.variables[mgr.ENV_DIR] == 'test/etc'


def test_upgrade_1():
    # Test with an explicit version, so that the test doesn't have to change on the next upgrade
    mgr = ConfigManager('cli', SCHEMA_1)

    mgr.load_yaml_string('''
verify_ssl: true
sites:
  example:
    verify_ssl: foobar.pem
    ''', 'test.yml')

    assert mgr['ssl_verify'] is True
    assert mgr['sites.example.ssl_verify'] == 'foobar.pem'

    with pytest.raises(KeyError):
        mgr['verify_ssl']

    fd = io.StringIO()
    mgr.dump(file_=fd)

    # the upgraded config should have a version
    assert "version: '1'" in fd.getvalue()


def test_upgrade_1_validate():
    mgr = ConfigManager('cli', SCHEMA_LATEST)

    with pytest.raises(ConfigurationError):
        mgr.load_yaml_string('''
bad_config: true
    ''', 'test.yml')



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2017 Continuum Analytics, Inc.
#
# All Rights Reserved.
# -----------------------------------------------------------------------------
import hashlib
import json
import os.path
import tarfile

CHUNK_SIZE = 16384


def extract_from_sh(infile):
    if isinstance(infile, str):
        f = open(infile, 'rb')
    else:
        f = infile
    text = f.read(100000)
    # Look for the start of the tarfile containing the main constructor
    # tarball, so the metadata can be extracted. In constructor 2, the
    # tarball started right after the @@END_HEADER@@. In version 3, we
    # added the standalone conda executable before that, so we need to
    # extract the offset and add that in.
    marker1 = b'@@END_HEADER@@'
    marker2 = b'boundary0 + '
    i1 = text.rfind(marker1) + len(marker1) + 1
    i2 = text.rfind(marker2)
    if i2 > 0:
        i2 += len(marker2)
        i2 = int(text[i2:i2 + 32].split()[0])
        i1 += i2
    f.seek(i1)
    tf = tarfile.open(fileobj=f, mode='r')
    efo = tf.extractfile('preconda.tar.bz2')
    tf2 = tarfile.open(fileobj=efo, mode='r:bz2')
    try:
        meta = json.loads(
            tf2.extractfile('pkgs/.constructor-build.info').read().decode())
    except KeyError:
        meta = {}
    meta['urls'] = tf2.extractfile('pkgs/urls').read().decode()
    return meta


def extract_from_ambari(infile):
    tf = tarfile.open(infile, mode='r:gz')
    _install_script = 'install.sh'
    for name in tf.getnames():
        if name.endswith(_install_script):
            break
    efo = tf.extractfile(name)
    return extract_from_sh(efo)


def extract_from_parcel(infile):
    tf = tarfile.open(infile, mode='r:gz')
    _install_script = 'meta/.constructor-build.info'
    for name in tf.getnames():
        if name.endswith(_install_script):
            break
    efo = tf.extractfile(name)
    meta = json.loads(efo.read().decode())
    return meta


def extract(infile):
    if infile.endswith('.sh'):
        out = extract_from_sh(infile)
        installer_format = 'constructor'
    elif infile.endswith('.parcel'):
        out = extract_from_parcel(infile)
        installer_format = 'parcel'
    elif infile.endswith('.tar.gz'):
        out = extract_from_ambari(infile)
        installer_format = 'mpack'
    else:
        raise ValueError('Installer type not known: %s' % infile)

    size = 0
    md5 = hashlib.md5()
    sha1 = hashlib.sha1()
    sha256 = hashlib.sha256()

    with open(infile, 'rb') as fd:
        for chunk in iter(lambda: fd.read(CHUNK_SIZE), b''):
            size += len(chunk)
            md5.update(chunk)
            sha1.update(chunk)
            sha256.update(chunk)

    return {
        'size': size,
        'md5': md5.hexdigest(),
        'sha1': sha1.hexdigest(),
        'sha256': sha256.hexdigest(),
        'filename': os.path.basename(infile),
        'manifest-attributes': out,
        'installer-format': installer_format,
    }



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
import click

from anaconda_enterprise.cli.api import create_client


@click.command(name='login')
@click.option('--username', prompt=True)
@click.password_option(confirmation_prompt=False)
@click.pass_obj  # object is a ConfigurationManager loaded with the callback on main
def main(config_manager, username, password):
    """Log in to the platform"""
    client = create_client(config_manager)
    client.login(username, password)
    click.echo('Logged %s in!' % (username, ))


@click.command(name='logout')
@click.pass_obj  # object is a ConfigurationManager loaded with the callback on main
def logout(config_manager):
    """Log out from the platform"""
    client = create_client(config_manager)
    client.token_store.delete_binstar_tokens()
    if config_manager.delete_token():
        click.echo('User logged out successfully.')
    else:
        click.echo("User already logged out: no active session found.")



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
import datetime
import os.path
import sys

import click

from anaconda_enterprise.cli import internal, parse
from anaconda_enterprise.cli.api import _list, create_client

ERROR_ = click.style('ERROR', fg='red')
SUCCESS_ = click.style('success', fg='green')


def _list_uploads(uploads, what='package'):
    _list(('Name', 'Uploaded (UTC)'), ('name', 'uploaded'),
          uploads,
          action='uploaded',
          what='package')


@click.command(name='upload')
@click.option('--channel', help="Upload to a specific channel")
@click.argument('package', type=click.Path(exists=True), nargs=-1)
@click.option('--ignore-name', help="Will not check the name of the file")
@click.pass_obj  # object in context is the Configuration Manager
def upload(config_manager, channel, package, ignore_name):
    """Upload a conda package"""
    sys.exit(_upload(config_manager, channel, package, ignore_name))


def _upload(config_manager, channel, package, ignore_name):
    """Upload a conda package"""
    try:
        default_channel = config_manager['default_channel']
    except KeyError:
        pass
    else:
        if channel is None:
            channel = default_channel
    client = create_client(config_manager)
    chan = client.get_channel(channel)
    if chan is None:
        click.echo('Error: invalid channel {0}.'.format(channel))
        return
    uploads = []
    for pkg in package:
        pkg_name = os.path.basename(pkg)
        try:
            click.echo('Uploading {0}...'.format(pkg_name), nl=False)
            attrs = parse.parse_conda(pkg, ignore_name_validation=ignore_name)
            upload = client.stage_file(chan, attrs)
            # upload icon file if it exists
            if 'has-icon' in attrs['recipe-attributes']:
                client.upload_icon(upload, pkg)
            # upload package file
            with open(pkg, 'rb') as fd:
                client.upload_file(upload, fd)
            uploads.append((pkg_name, datetime.datetime.utcnow().isoformat()))
            click.echo(SUCCESS_)
        except (internal.ServerError, ValueError,
                internal.JSONAPIError) as exc:
            exc = str(exc).replace('\n', '\n| ')
            click.echo('{0}\n| {1}'.format(ERROR_, exc))
    if uploads:
        _list_uploads(uploads)
    else:
        click.echo('No packages uploaded.')
    return 0 if len(uploads) == len(package) else 1


@click.command(name='upload_pypi')
@click.option('--pypi-index', help="Upload to a specific index", required=True)
@click.argument('package', type=click.Path(exists=True), nargs=-1)
@click.option('--ignore-name', help="Will not check the name of the file")
@click.pass_obj  # object in context is the Configuration Manager
def upload_pypi(config_manager, pypi_index, package, ignore_name):
    """Upload a pypi package"""
    client = create_client(config_manager)
    pypi_index = client.get_pypi_index(pypi_index)
    uploads = []
    for pkg in package:
        pkg_name = os.path.basename(pkg)
        try:
            click.echo('Uploading {0}...'.format(pkg_name), nl=False)
            attrs = parse.parse_pypi(pkg)
            upload = client.stage_pypi_file(pypi_index, attrs)
            with open(pkg, 'rb') as fd:
                client.upload_file(upload, fd)
            uploads.append((pkg_name, datetime.datetime.utcnow().isoformat()))
            click.echo(SUCCESS_)
        except (internal.ServerError, ValueError,
                internal.JSONAPIError) as exc:
            exc = str(exc).replace('\n', '\n| ')
            click.echo('{0}\n| {1}'.format(ERROR_, exc))
    if uploads:
        _list_uploads(uploads)
    else:
        click.echo('No packages uploaded.')
    sys.exit(0 if len(uploads) == len(package) else 1)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

import pytest

from anaconda_enterprise.cli.api import ApiClient, create_client
from anaconda_enterprise.cli.internal import ClientError


def _setUp(mocker, expected_acls):
    mocker.patch.object(ApiClient, 'check')
    ApiClient.check.return_value = None

    mocker.patch.object(ApiClient, 'channels')
    ApiClient.channels.return_value = [{'id': 1, 'attributes': {'name': 'foo', 'data': 1}}]

    mocker.patch.object(ApiClient, '_json_api')
    ApiClient._json_api.return_value = expected_acls


def test_get_channel_acls(config_manager, mocker):
    expected_acls = [{'id': 1, 'attributes': {'name': 'mike', 'namespace': 'users'}}]
    _setUp(mocker, expected_acls)

    client = create_client(config_manager)
    acls = client.get_channel_acls('foo')

    assert acls is not None
    assert acls[0]['id'] == 1


def test_get_channel_acls_not_found(config_manager, mocker):
    expected_acls = [{'id': 1, 'attributes': {'name': 'mike', 'namespace': 'users'}}]
    _setUp(mocker, expected_acls)

    client = create_client(config_manager)
    acls = client.get_channel_acls('bar')

    assert acls is None


def test_find_channel_acl(config_manager, mocker):
    expected_acls = [{'id': 1, 'attributes': {'name': 'mike', 'namespace': 'users'}}]
    _setUp(mocker, expected_acls)

    client = create_client(config_manager)
    acl = client.find_channel_acl('foo', 'mike', 'users')

    assert acl is not None
    assert acl['id'] == expected_acls[0]['id']
    assert acl['attributes']['name'] == expected_acls[0]['attributes']['name']
    assert acl['attributes']['namespace'] == expected_acls[0]['attributes']['namespace']


def test_find_channel_acl_not_found(config_manager, mocker):
    expected_acls = [{'id': 1, 'attributes': {'name': 'mike', 'namespace': 'users'}}]
    _setUp(mocker, expected_acls)

    client = create_client(config_manager)
    acl = client.find_channel_acl('foo', 'rami', 'users')

    assert acl is None


def test_remove_shared_channel(config_manager, mocker):
    expected_acls = [{'id': 1, 'attributes': {'name': 'mike', 'namespace': 'users'}}]
    _setUp(mocker, expected_acls)

    client = create_client(config_manager)
    client.remove_shared_channel('foo', 'mike', 'users')

    ApiClient._json_api.return_value = []

    acl = client.find_channel_acl('foo', 'mike', 'users')

    assert acl is None


def test_remove_shared_channel_not_found(config_manager, mocker):
    expected_acls = [{'id': 1, 'attributes': {'name': 'mike', 'namespace': 'users'}}]
    _setUp(mocker, expected_acls)

    client = create_client(config_manager)
    with pytest.raises(ClientError):
        client.remove_shared_channel('foo', 'rami', 'users')

    acl = client.find_channel_acl('foo', 'mike', 'users')
    assert acl is not None



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2018 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
import os.path
import re

from click.testing import CliRunner
import pytest

from .base import VCR, FIXTURES
from anaconda_enterprise.cli.main import group

pkg_dir = os.path.join(FIXTURES, "conda-mirror", "linux-64")


@pytest.mark.integration
@VCR.use_cassette()
def test_packages_list_and_delete_single_file():
    runner = CliRunner()
    result = runner.invoke(group, ['login'], 'derek\nderek')

    result = runner.invoke(group, ['channels', 'create', 'test1'])

    result = runner.invoke(group, [
        'upload', '--channel', 'test1', os.path.join(pkg_dir, "affine-2.0.0-py27_0.tar.bz2"),
        os.path.join(pkg_dir, "affine-2.0.0-py34_0.tar.bz2")
    ])

    assert result.exit_code == 0

    result = runner.invoke(
        group,
        ['packages', 'list-files', 'test1', 'affine'], )

    assert len(re.findall('affine-2.0.0', result.output)) == 2
    assert result.exit_code == 0

    result = runner.invoke(
        group,
        [
            'packages', 'delete', 'test1', 'affine', '--file',
            'linux-64/affine-2.0.0-py27_0.tar.bz2'
        ],
        input='Y\n', )
    assert result.exit_code == 0

    result = runner.invoke(
        group,
        ['packages', 'list-files', 'test1', 'affine'], )

    assert len(re.findall('affine-2.0.0', result.output)) == 1
    assert len(re.findall('affine-2.0.0-py34_0.tar.bz2', result.output)) == 1
    assert result.exit_code == 0


@pytest.mark.integration
@VCR.use_cassette()
def test_packages_list_and_delete_package():
    runner = CliRunner()
    result = runner.invoke(group, ['login'], 'derek\nderek')

    result = runner.invoke(group, ['channels', 'create', 'test1'])

    result = runner.invoke(
        group,
        ['upload', '--channel', 'test1', os.path.join(pkg_dir, "alabaster-0.7.7-py27_0.tar.bz2")])

    assert result.exit_code == 0

    result = runner.invoke(
        group,
        ['packages', 'list-files', 'test1', 'alabaster'], )

    assert 'alabaster-0.7.7-py27_0.tar.bz2' in result.output
    assert result.exit_code == 0

    result = runner.invoke(
        group,
        [
            'packages', 'delete', 'test1', 'alabaster', '--file',
            'linux-ppc64le/alabaster-0.7.7-py27_0.tar.bz2'
        ],
        input='Y\n', )
    assert result.exit_code == 0

    result = runner.invoke(
        group,
        ['packages', 'list-files', 'test1', 'alabaster'], )

    assert 'Error: Package "test1/alabaster" was not found\n' == result.output
    assert result.exit_code == 1



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
import sys

import click

try:
    from ruamel.yaml import YAML
except ImportError:
    from ruamel_yaml import YAML

yaml = YAML(typ='safe')
yaml.default_flow_style = False


class Unauthorized(click.ClickException):
    """User is not authorized"""


class JSONAPIError(click.ClickException):
    """A JSON API error container"""
    def __init__(self, errors):
        self.status = errors[0]['status']
        message = ', '.join(error['detail'] for error in errors)
        super(JSONAPIError, self).__init__(message)


class ServerError(click.ClickException):
    """Server error"""


class ClientError(click.ClickException):
    """Client error"""


def stdin_is_interactive():
    """True if stdin is a tty."""
    return sys.stdin.isatty()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# All Rights Reserved.
# -----------------------------------------------------------------------------
import logging
import sys

import click

from anaconda_enterprise.cli import __version__
from anaconda_enterprise.cli.commands import (admin, channels, config_cmd,
                                              login, packages, pypi_index,
                                              spark_config, upload)
from anaconda_enterprise.cli.config import load_config
from anaconda_enterprise.cli.internal import stdin_is_interactive

program_name = 'anaconda-enterprise-cli'
slugified_program_name = program_name.replace('-', '_')
log = logging.getLogger(program_name)


# Click 7.0 introduced a breaking change that requires subcommands
# to use dashes instead of underscores. This provides back compatibility
# https://click.palletsprojects.com/en/7.x/upgrading/#upgrading-to-7-0
def normalize_command(name):
    return name.replace('_', '-')


group = click.Group(
    program_name, context_settings={'token_normalize_func': normalize_command})
click.version_option(__version__, prog_name=program_name)(group)
group.add_command(admin.main)
group.add_command(channels.main)
group.add_command(config_cmd.main)
group.add_command(login.main)
group.add_command(login.logout)
group.add_command(packages.main)
group.add_command(pypi_index.main)
group.add_command(upload.upload)
group.add_command(upload.upload_pypi)
group.add_command(spark_config.main)
group.add_command(spark_config.main, name='site-config')


@click.pass_context
def load_configuration_manager(context):
    config = load_config()
    context.obj = config


group.callback = load_configuration_manager


def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        group.main(argv[1:], prog_name=program_name, standalone_mode=True)
    except (KeyboardInterrupt, click.Abort):
        sys.stderr.write('Aborted!')
        return 1
    except click.ClickException as exc:
        exc.show()
        return exc.exit_code
    except Exception:
        # Unexpected error
        (exception_type, exception_value, exception_trace) = sys.exc_info()

        try:
            import datetime
            import tempfile
            import traceback

            print("An unexpected error occurred, most likely a bug in %s." %
                  program_name,
                  file=sys.stderr)
            print("    (The error was: %s: %s)" %
                  (exception_type.__name__, str(exception_value)),
                  file=sys.stderr)

            # so batch jobs have the details in their logs
            output_to_console = not stdin_is_interactive()

            when = datetime.date.today().isoformat()
            prefix = "bug_details_%s_%s_" % (slugified_program_name, when)
            with tempfile.NamedTemporaryFile(prefix=prefix,
                                             suffix=".txt",
                                             delete=False) as bugfile:
                report_name = bugfile.name

                def output(s):
                    bugfile.write(s.encode('utf-8'))
                    bugfile.write("\n".encode('utf-8'))
                    if output_to_console:
                        print(s, file=sys.stderr)

                output("Bug details for %s error on %s" % (program_name, when))
                output("")
                output("sys.argv: %r" % sys.argv)
                output("")
                output("")
                output("\n".join(
                    traceback.format_exception(exception_type, exception_value,
                                               exception_trace)))

            if output_to_console:
                print("Above details were also saved to %s" % report_name,
                      file=sys.stderr)
            else:
                print("Details about the error were saved to %s" % report_name,
                      file=sys.stderr)
        except Exception:
            # re-raise the original exception, which is probably more useful
            # than reporting whatever was broken about our bug handling code
            # above.
            raise exception_value

        # exit code
        return 1


if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
from os.path import dirname, join

import vcr

FIXTURES = join(dirname(__file__), 'fixtures')
VCR = vcr.VCR(cassette_library_dir=join(FIXTURES, 'cassettes'))



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
import base64
import hashlib
import json
import os

import click

from anaconda_enterprise.cli.internal import yaml

ERROR_ = click.style('ERROR', fg='red')


@click.command(name='spark-config')
@click.option('--config',
              multiple=True,
              type=(str, str),
              help="Config Tuple: (path in project, local path)")
@click.option('--output',
              type=str,
              help="Output YAML",
              default="anaconda-config-files-secret.yaml")
def main(config, output):
    """Manage configuration files for the user Docker image

       This command manages a special Kubernetes secret used by AE5 to
       customize the user Docker image by inserting configuration files
       upon startup. Historically, this command was called "spark-config"
       because of its use to configure connections to Spark clusters.
       Because of its wider utility, it can now be called "site-config" as well.
    """

    files_dict = {}
    for rp, lp in config:
        if not os.path.exists(lp):
            click.echo('Error: invalid local path {0}.'.format(lp))
            return
        else:
            with open(lp, 'rb') as f:
                data = f.read()
            files_dict[rp] = data

    yaml_config = make_config(files_dict)

    click.echo("Writing secret data to %s with the following paths: " % output)
    click.echo("\t%s " % '\n\t'.join(files_dict.keys()))

    with open(output, 'w') as f:
        f.write(yaml_config)

    click.echo("Completed...\nPlease upload secret to Kubernetes:\n"
               ">>> sudo kubectl replace -f %s " % output)


def _encode_str(data):
    try:
        return str(base64.b64encode(data), encoding='utf-8')
    except Exception:
        return str(base64.b64encode(data)).encode('UTF-8')


def make_config(files_dict):
    files_data = {
        hashlib.md5(path.encode('utf8')).hexdigest(): _encode_str(value)
        for (path, value) in files_dict.items()
    }
    manifest = {
        hashlib.md5(path.encode('utf8')).hexdigest(): path
        for path in files_dict
    }
    files_data['anaconda_manifest'] = _encode_str(
        json.dumps(manifest).encode('utf8'))
    return yaml.dump({
        'apiVersion': 'v1',
        'metadata': {
            'name': 'anaconda-config-files'
        },
        'data': files_data,
        'kind': 'Secret',
        'type': 'Opaque'
    })



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2017 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
from click.testing import CliRunner
import pytest

from anaconda_enterprise.cli.main import group


def test_set_channel(config):  # config is a bad name, but see conftests.py
    runner = CliRunner()
    result = runner.invoke(group, ['config', 'set', 'default_channel', 'channel123'])
    assert result.exit_code == 0
    assert "default_channel: channel123\n" in config.join('cli.yml').read()


def test_set_channel_config_file_doesnot_exist(
        config):  # config is a bad name, but see conftests.py
    runner = CliRunner()
    config.join('cli.yml').remove()
    result = runner.invoke(group, ['config', 'set', 'default_channel', 'channel123'])
    assert result.exit_code == 0
    assert "default_channel: channel123\n" in config.join('cli.yml').read()


def test_get_config_not_set(config):
    runner = CliRunner()
    # conf file exists, but no default_channel see conftests.py
    result = runner.invoke(group, ['config', 'get', 'default_channel'])
    assert result.exit_code != 0
    assert result.output == "Error: Option 'default_channel' is not set\n"


def test_get_channel_config_file_doesnot_exist(
        config):  # config is a bad name, but see conftests.py
    runner = CliRunner()
    config.join('cli.yml').remove()
    result = runner.invoke(group, ['config', 'get', 'default_channel'])
    assert result.exit_code != 0
    assert result.output == "Error: Option 'default_channel' is not set\n"


def test_get_existing_config(config):
    runner = CliRunner()
    # conf file exists
    config.join('cli.yml').write("default_channel: channel123\n", mode="a")
    result = runner.invoke(group, ['config', 'get', 'default_channel'])
    assert result.exit_code == 0
    assert result.output == "'channel123'\n"


def test_set_channel_with_existing_config(config):
    runner = CliRunner()
    # conf file exists
    config.join('cli.yml').write("default_channel: channel123\n", mode="a")
    result = runner.invoke(group, ['config', 'set', 'default_channel', 'channel456'])
    assert result.exit_code == 0
    assert "default_channel: channel456\n" in config.join('cli.yml').read()


def test_set_channel_invalid_chars_in_name(config):
    runner = CliRunner()
    result = runner.invoke(group, ['config', 'set', 'default_channel', 'cha!nnel1'])
    assert result.exit_code != 0
    assert result.output == \
        ("Error: validating configuration 'default_channel'"
         ", 'cha!nnel1' does not match '^[%@/0-9A-Za-z_-]+$'\n")


def test_set_channel_with_underscore(config):
    runner = CliRunner()
    result = runner.invoke(group, ['config', 'set', 'default_channel', 'channel_one'])
    assert result.exit_code == 0
    assert "default_channel: channel_one\n" in config.join('cli.yml').read()


def test_set_channel_with_dashes(config):
    runner = CliRunner()
    result = runner.invoke(group, ['config', 'set', 'default_channel', 'channel-one'])
    assert result.exit_code == 0
    assert "default_channel: channel-one\n" in config.join('cli.yml').read()


def test_set_non_existing_config(config):
    runner = CliRunner()
    result = runner.invoke(group, ['config', 'set', 'anything', 'False'])
    assert result.exit_code != 0
    assert result.output == \
        ("Error: validating configuration '', Additional properties are"
         " not allowed ('anything' was unexpected)\n")


def test_get_ssl_verify(config_v0):
    runner = CliRunner()
    result = runner.invoke(group, ['config', 'get', 'default_site'])
    assert result.exit_code == 0
    assert result.output == "'example'\n"


def test_set_config_w_multiple_types(config):
    runner = CliRunner()
    # We are using 'sites.test.ssl_verify' as attribute because ssl_verify is already
    # used as a mock value in the file created by tests coftest.py . Check that file
    # for more details
    result = runner.invoke(group, ['config', 'set', 'sites.test.ssl_verify', 'False'])
    assert result.exit_code == 0
    config_content = config.join('cli.yml').read()
    assert "ssl_verify: false\n" in config_content


def test_disallow_set_config_sites_as_string(config):
    runner = CliRunner()
    site_to_set = 'https://enterprise.demo.continuum.io/repository/api'
    assert type(site_to_set) == str
    pre_result = runner.invoke(group, ['config', 'view']).output
    result = runner.invoke(group, ['config', 'set', 'sites', site_to_set])
    assert result.exit_code != 0
    assert result.output == (
        "Error: validating configuration 'sites', '{s}' is not of type 'object'\n".format(
            s=site_to_set))
    post_result = runner.invoke(group, ['config', 'view']).output
    assert pre_result == post_result


def test_set_sites_with_dict(config):
    runner = CliRunner()
    sites = dict()
    result = runner.invoke(group, ['config', 'set', 'sites', str(sites)])
    assert result.exit_code == 0



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
import os
import pytest

from anaconda_enterprise.cli.config import ConfigManager, platform_defaults, SCHEMA_LATEST

paths = platform_defaults + ('{root}/{component_file}', )
CA_CHAIN = os.environ.get('CA_CHAIN', os.path.join('..', 'build', 'certs', 'rootca.crt'))


class TestConfigManager(ConfigManager):
    def __init__(self, component, version, path):
        super(TestConfigManager, self).__init__(component, version)
        self.variables['root'] = path

    @property
    def user_local_save_path(self):
        return "{root}/{component_file}".format(**self.variables)

    def get_search_path(self):
        return [path.format(**self.variables) for path in paths]

    @property
    def token_path(self):
        return '{root}/tokens.json'.format(**self.variables)


@pytest.fixture()
def config_manager(monkeypatch, tmpdir):
    root = tmpdir.mkdir('cli-root-cm')

    root.join('cli.yml').write('''
version: '{version}'
default_site: example
sites:
  example:
    url: "https://repo.anaconda.example.com:8089/api"
    ssl_verify: {ca_chain}
'''.format(
        version=SCHEMA_LATEST, ca_chain=CA_CHAIN))
    mgr = TestConfigManager('cli', SCHEMA_LATEST, str(root))
    mgr.load_schema()
    mgr.load_files()
    monkeypatch.setattr('anaconda_enterprise.cli.config.config', mgr)
    return mgr


@pytest.fixture(autouse=True)
def config(monkeypatch, tmpdir):
    root = tmpdir.mkdir('cli-root')

    root.join('cli.yml').write('''
version: '{version}'
default_site: example
sites:
  example:
    url: "https://anaconda.example.com/repository/api"
    ssl_verify: {ca_chain}
'''.format(
        version=SCHEMA_LATEST, ca_chain=CA_CHAIN))

    mgr = TestConfigManager('cli', SCHEMA_LATEST, str(root))
    monkeypatch.setattr('anaconda_enterprise.cli.config.config', mgr)
    return root


@pytest.fixture()
def config_v0(monkeypatch, tmpdir):
    root = tmpdir.mkdir('cli0-root')

    root.join('cli.yml').write('''
default_site: example
sites:
  example:
    url: "https://anaconda.example.com/repository/api"
    verify_ssl: {ca_chain}
'''.format(ca_chain=CA_CHAIN))

    mgr = TestConfigManager('cli', SCHEMA_LATEST, str(root))
    monkeypatch.setattr('anaconda_enterprise.cli.config.config', mgr)
    return root
